// Autogenerated AST node
package org.python.pydev.parser.jython.ast;

public interface expr_contextType {
    public static final int Load = 1;
    public static final int Store = 2;
    public static final int Del = 3;
    public static final int AugLoad = 4;
    public static final int AugStore = 5;
    public static final int Param = 6;
    public static final int Artificial = 7;
    public static final int KwOnlyParam = 8;

    public static final String[] expr_contextTypeNames = new String[] { "<undef>", "Load", "Store", "Del", "AugLoad",
            "AugStore", "Param", "Artificial", "KwOnlyParam", };
}
