// Autogenerated AST node
package org.python.pydev.parser.jython.ast;

public interface comp_contextType {
    public static final int ListCtx = 1;
    public static final int TupleCtx = 2;
    public static final int EmptyCtx = 3;

    public static final String[] comp_contextTypeNames = new String[] { "<undef>", "ListCtx", "TupleCtx", "EmptyCtx", };
}
