/**
 * Copyright (c) 2005-2011 by Appcelerator, Inc. All Rights Reserved.
 * Licensed under the terms of the Eclipse Public License (EPL).
 * Please see the license.txt included with this distribution for details.
 * Any modifications to this file must keep this entire header intact.
 */
/*
 * Created on Apr 3, 2006
 */
package org.python.pydev.parser.visitors.comparator;

public class DifferException extends Exception {

    public DifferException(String string) {
        super(string);
    }

}
